       identification division.
       program-id. 20210216.

       environment division.

       input-output section.
           file-control.

       data division.

       file section.

       working-storage section.

       01  ws-num-enemies                pic 9 value 5.

       01  ws-game-state.
           05  ws-is-gameover-sw         pic x value 'N'.
               88  ws-is-gameover        value 'Y'.
               88  ws-not-gameover       value 'N'.

       01  ws-shared-char-attrs.
           05  ws-char-mask              pic x value space.
           05  ws-char-len               pic x(2) comp-x.
       
       01  ws-player.
           05  ws-player-health          pic 9(3) value 100.
           05  ws-player-char            pic x value "$".
           05  ws-player-pos.
               10  ws-player-y           pic x comp-x.
               10  ws-player-x           pic x comp-x.               

       01  ws-enemy occurs 1 to 5 times depending on ws-num-enemies.
           05  ws-enemy-moved-sw         pic x value 'N'.
               88  ws-enemy-moved        value 'Y'.
               88  ws-enemy-not-moved    value 'N'.
           05  ws-enemy-attack-sw        pic x value 'N'.
               88  ws-enemy-attacked     value 'Y'.
               88  ws-enemy-not-attacked value 'N'.               
           05  ws-enemy-char             pic x value "@".
           05  ws-enemy-pos.
               10  ws-enemy-y            pic x comp-x.
               10  ws-enemy-x            pic x comp-x.
           05  ws-moved-this-tick        pic 9. 

      *> Temp storage to set what to draw/mask
       01  ws-temp-obj.
           05  ws-obj-char               pic x value space.
           05  ws-obj-pos.
               10  ws-obj-y              pic x comp-x.
               10  ws-obj-x              pic x comp-x.
                  
       01  ws-screen-pos.
           05  ws-row                    pic x comp-x.
           05  ws-col                    pic x comp-x.
       
       01  ws-clear-scr-params.
           05  ws-clr-char               pic x comp-x.
           05  ws-clr-attr               pic x comp-x.
    
       01  ws-user-input.
           05  ws-key-status             pic x comp-x.   
           05  ws-key-in                 pic x.
           05  ws-accept                 pic x.
       
       
       01  ws-position-delta.
           05  ws-delta-y                pic x comp-x.
           05  ws-delta-x                pic x comp-x.
           
       01  ws-return-code                pic xx.

       01  ws-idx                        pic 9.

       01  ws-date-time occurs 2 times.
           05  ws-date        pic 9(8).
           05  ws-hour        pic 9(2).
           05  ws-min         pic 9(2).
           05  ws-sec         pic 9(2).
           05  ws-milli       pic 9(2).
       
       01  ws-start-date-time            pic 9(6).
       01  ws-end-date-time              pic 9(6).
       01  ws-fps                        pic 99.

       screen section.

       procedure division.

       init.
           move 1 to ws-char-len
           move 15 to ws-player-x
           move 15 to ws-player-y
       
           move 10 to ws-clr-attr

           perform varying ws-idx
           from 1 by 1 until ws-idx > ws-num-enemies               
               compute ws-enemy-x(ws-idx) = ws-idx * 14
               compute ws-enemy-y(ws-idx) = ws-idx * 4 + 10
           end-perform
      
           move 1 to ws-row
           move zero to ws-col.
       
       main-procedure.
      *> NOTE: CANNOT DO BELOW CALL. MUST SPECIFY EACH PARAM!!!
      *> call "CBL_CLEAR_SCR" using ws-clear-scr-params
      *>     call X"E4" <-- clear screen call directly
           call "CBL_CLEAR_SCR" using ws-clr-char ws-clr-attr
           call "CBL_SET_CSR_POS" using ws-screen-pos
           display "Test Game - 2021-02-16"
       
           perform draw-player-health

           move ws-player-pos to ws-obj-pos
           move ws-player-char to ws-obj-char
           perform draw-temp-obj

           perform until ws-is-gameover
               move function current-date to ws-date-time(1)

               perform input-handler   
               perform enemy-handler
               perform collision-handler
        
               if ws-player-health is zero then
                   set ws-is-gameover to true
                   perform display-game-over
               end-if
               
               move zero to ws-fps
               perform until ws-fps > zero
                   move function current-date to ws-date-time(2)
                   compute ws-fps = 
                           ws-milli(2) - ws-milli(1)
               end-perform
               move 10 to ws-row
               move 1 to ws-col
               call "CBL_SET_CSR_POS" using ws-screen-pos
               display "WS-FPS: " ws-fps

           end-perform     
           accept ws-key-in with timeout after 5
           move 7 to ws-clr-attr
           call "CBL_CLEAR_SCR" using ws-clr-char ws-clr-attr
       
           stop run.


       draw-player-health.
           move 1 to ws-row
           move 60 to ws-col.
           call "CBL_SET_CSR_POS" using ws-screen-pos
           display "HEALTH: " ws-player-health
           exit paragraph.

       draw-temp-obj.
           call "CBL_WRITE_SCR_CHARS" using 
            ws-obj-pos ws-obj-char ws-char-len
           exit paragraph.
       

       input-handler.
      *> Getting KB input will set cursor position. So reset to top of
      *> screen so it doesn't scroll off the screen.
           move zero to ws-row
           move zero to ws-col
           call "CBL_SET_CSR_POS" using ws-screen-pos
           
      *> Make sure there's a key press to be acted upon before blocking
      *> game loop reading character input.        
           call "CBL_GET_KBD_STATUS" using ws-key-status 
               returning ws-return-code                       
           if ws-key-status = zeros then
               exit paragraph
           end-if
           
           call "CBL_READ_KBD_CHAR" using ws-key-in
       
           move zero to ws-delta-y
           move zero to ws-delta-x
       
           evaluate ws-key-in
               when 'q'
                   set ws-is-gameover to true
               when 'M'            
                   add 1 to ws-delta-x
               when 'K'
                   subtract 1 from ws-delta-x
               when 'H' 
                   subtract 1 from ws-delta-y
               when 'P'
                   add 1 to ws-delta-y
            
               when other
                   display ws-key-in
           end-evaluate
           
      *> If player moved, mask current location and redraw at new location
           if ws-delta-x <> 0 or ws-delta-y <> 0 then
               move ws-player-pos to ws-obj-pos
               move space to ws-obj-char
               perform draw-temp-obj
        
               add ws-delta-x to ws-player-x
               add ws-delta-y to ws-player-y
        
               move ws-player-pos to ws-obj-pos
               move ws-player-char to ws-obj-char
               perform draw-temp-obj
           end-if
       
           exit paragraph.
       
       enemy-handler.
              
           perform varying ws-idx
           from 1 by 1 until ws-idx > ws-num-enemies
       
               move ws-enemy-pos(ws-idx) to ws-obj-pos
               move ws-enemy-char(ws-idx) to ws-obj-char
               perform draw-temp-obj

      *> TODO : make this more configurable to move enemy on game ticks.
               if ws-milli(1)(1:1) = zero then
                   set ws-enemy-not-moved(ws-idx) to true
               end-if
          
               if ws-enemy-not-moved(ws-idx) then           

                   move ws-enemy-pos(ws-idx) to ws-obj-pos
                   move space to ws-obj-char
                   perform draw-temp-obj
                   
                   if ws-player-y > ws-enemy-y(ws-idx) then
                       add 1 to ws-enemy-y(ws-idx)
                       set ws-enemy-moved(ws-idx) to true
                   end-if
                   if ws-player-y < ws-enemy-y(ws-idx) then
                       subtract 1 from ws-enemy-y(ws-idx)
                       set ws-enemy-moved(ws-idx) to true
                   end-if
                   if ws-player-x > ws-enemy-x(ws-idx) then
                       add 1 to ws-enemy-x(ws-idx)
                       set ws-enemy-moved(ws-idx) to true
                   end-if
                   if ws-player-x < ws-enemy-x(ws-idx) then
                       subtract 1 from ws-enemy-x(ws-idx)
                       set ws-enemy-moved(ws-idx) to true
                   end-if

                   move ws-enemy-pos(ws-idx) to ws-obj-pos
                   move ws-enemy-char(ws-idx) to ws-obj-char
                   perform draw-temp-obj
       
               end-if
           end-perform
       
           exit paragraph.     


       collision-handler.
       
           perform varying ws-idx
           from 1 by 1 until ws-idx > ws-num-enemies
       
               if ws-enemy-attacked(ws-idx) then

                   if ws-sec(1)(2:1) = zero then
      *>                 display "ATTACK RESET on " ws-idx
                       set ws-enemy-not-attacked(ws-idx) to true
                   end-if
                   exit paragraph
               end-if
           
               if ws-player-x = ws-enemy-x(ws-idx)
                   and ws-player-y = ws-enemy-y(ws-idx) then

                   move ws-player-pos to ws-obj-pos
                   move "X" to ws-obj-char
                   perform draw-temp-obj

                   if ws-player-health >= 5 then
                       subtract 5 from ws-player-health
                       perform draw-player-health
                   end-if
            
                   set ws-enemy-attacked(ws-idx) to true
               end-if
           end-perform
           
           exit paragraph.
       
       
       display-game-over.
           move 15 to ws-row
           move 20 to ws-col.
           call "CBL_SET_CSR_POS" using ws-screen-pos
       
           display "You've died! Game over!"
       
           move 16 to ws-row
           move 22 to ws-col.
           call "CBL_SET_CSR_POS" using ws-screen-pos
           display "Press Enter to exit"
       
           accept ws-accept
           exit paragraph.

       end program 20210216.       








