@echo off
REM *---------------------------------------------------------------------------*
REM *   Microsoft COBOL Build Script                                            *
REM *---------------------------------------------------------------------------*
REM * Please set variables below as needed.                                     *
REM *                                                                           *
REM *---------------------------------------------------------------------------*
REM *   Script variables:                                                       *
REM *     * File name ".CBL" is assumed on PROGNAME                             *
REM *     * PROGRAM-ID in source file must match actual file name for animator  *
REM *       debugger to read the listing file.                                  *
REM *     * Uncomment out EXTRAOBJS variable if needed.                         *
REM *---------------------------------------------------------------------------*

SET PROGNAME=20210216
SET REQLIB=FALSE
SET ANIMDEBUG=FALSE
SET RUNPROG=true

REM Add extra library objects here (found in c:\cobol\lib). Separate with a "+".
SET EXTRAOBJS=+COBINTFN

REM *---------------------------------------------------------------------------*
REM * NOTE: If you get an error for "Out of environment space", that means you  *
REM *       potentially have too many variables set in your autoexec.bat.       *
REM *                                                                           *
REM *   Possile solutions:                                                      *
REM *       * In build script, replace variables with hardcoded values instead. *
REM *       * Comment out unneeded SET commands in AUTOEXEC.BAT                 *
REM *       * Update CONFIG.SYS to use /e:ENVSIZE command line switch           *
REM *         Example: SHELL=C:\COMMAND.COM /e:1024 /p                          *
REM *       * If running in windows, update SYSTEM.INI to increase              *
REM *         environment size in command prompt window.                        *
REM *         * Under section "[NonWindowsApp]" add:                            *
REM *           CommandEnvSize=1024                                             *
REM *                                                                           *
REM *   (Adjust environement size parameters as needed)                         *
REM *---------------------------------------------------------------------------*

if not exist %PROGNAME%.CBL goto errbuild
if [%PROGNAME%] == [] goto errbuild

if %ANIMDEBUG% == TRUE goto doanim
if %ANIMDEBUG% == true goto doanim

echo     *---------------------------------------------------------------------*
echo     *          Compiling Program: %PROGNAME% 
echo     *---------------------------------------------------------------------*
@echo on
COBOL %PROGNAME%.CBL;
@echo off

if errorlevel 1 goto nocob
echo     *---------------------------------------------------------------------*
echo     *          Compiling successfully completed                           *
echo     *---------------------------------------------------------------------*


echo     *---------------------------------------------------------------------*
echo     *          Linking Program: %PROGNAME%
echo     *                                                                     *
echo     *                                                                     *
if %REQLIB% == TRUE goto ltxtl
if %REQLIB% == true goto ltxtl

echo     *  The program will be linked to run with the shared run-time,        *
echo     *  COBLIB. The EXE file created requires the file COBLIB.DLE to be    *
echo     *  present in the COBOL system directories in order to operate.       *

goto ltxte

:ltxtl
echo     *  The program will be statically linked. That is, the COBOL run-time *
echo     *  support required for this program is linked into the EXE file      *
echo     *  making it independent of any other files at run-time.              *

:ltxte
echo     *                                                                     *
echo     *---------------------------------------------------------------------*

if %REQLIB% == TRUE goto linkl
if %REQLIB% == true goto linkl

:linkc
@echo on
LINK %PROGNAME%.OBJ%EXTRAOBJS%/NOD,%PROGNAME%.EXE,,COBLIB+COBAPI;
@echo off
if errorlevel 1 goto nolink
goto linke

:linkl
@echo on
LINK %PROGNAME%.OBJ%EXTRAOBJS%/NOD,%PROGNAME%.EXE,,LCOBOL+COBAPI;
@echo off
if errorlevel 1 goto nolink

:linke
@echo off
echo     *---------------------------------------------------------------------*
echo     *          Linking successfully completed                             *
echo     *---------------------------------------------------------------------*

if %RUNPROG% == FALSE goto endbuild
if %RUNPROG% == false goto endbuild
echo     *---------------------------------------------------------------------*
echo     *          Launching Program: %PROGNAME%
echo     *---------------------------------------------------------------------*
@echo on
%PROGNAME%
@echo off
goto endbuild

:doanim
REM ----------------------------------
REM  Build and run animator debugger
REM ----------------------------------
echo     *---------------------------------------------------------------------*
echo     *          Compiling the %PROGNAME%.CBL for Animation debugging
echo     *---------------------------------------------------------------------*
@echo on
COBOL %PROGNAME%.CBL ANIM;
@echo off
if errorlevel 1 goto nocob
echo     *---------------------------------------------------------------------*
echo     *          Compiling successfully completed                           *
echo     *---------------------------------------------------------------------*
echo     *---------------------------------------------------------------------*
echo     *          Animating: %PROGNAME%
echo     *---------------------------------------------------------------------*
@echo on
ANIMATE %PROGNAME%
@echo off
if errorlevel 1 goto nocob
echo     *---------------------------------------------------------------------*
echo     *          Animating successfully completed                           *
echo     *---------------------------------------------------------------------*
goto endbuild


REM -----------------
REM Error messages
REM -----------------
:nocob
echo     ***********************************************************************
echo     *                                                                     *
echo     * An error occurred while running the Compiler. Please check output   *
echo     * for error information.                                              *
echo     *                                                                     *
echo     ***********************************************************************
goto endbuild

:nolink
echo     ***********************************************************************
echo     *                                                                     *
echo     * An error occurred while running the Linker. Please check output     *
echo     * for error information.                                              *
echo     *                                                                     *
echo     ***********************************************************************
goto endbuild

:errbuild
echo     ***********************************************************************
echo     *                                                                     *
echo     * The file %PROGNAME%.CBL is not in the current directory.            *
echo     *                                                                     *
echo     ***********************************************************************


:endbuild




